# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Fri Feb 24 16:00:00 2017

Modifications
-------------
-

Aims
----
- Song class

"""

import os
import glob
import sys

import numpy as np
from astropy.io import fits
from tqdm import trange
from joblib import Parallel, delayed, dump, load
from astropy.table import Table
from .utils import scan_files
import astropy.units as u
from .master import combine_image
from skimage.morphology import disk
from twodspec import ccdproc_mod as ccdproc
from twodspec.aperture import (combine_apertures, group_apertures,
                               substract_scattered_light, apflatten)


ALL_IMGTYPE = {
    "BIAS", "FLAT", "FLATI2", "THAR", "THARI2", "STAR", "STARI2", "TEST"}


class Config(object):
    kwds = dict(
        kw_exptime="EXPTIME",
        kw_pregain="PRE_GAIN",
        kw_gain="PRE_GAIN",  # key word for GAIN
        kw_imagetype="IMAGETYP",
        kw_object="OBJECT",
        kw_objname="OBJ-NAME",
        kw_slit="SLIT",
        kw_i2pos="I2POS",
        kw_mjdmid="MJD-MID",
        kw_bvc="BVC",
        kw_filepath="fps",
    )
    read = dict(
        hdu=0,
        unit="adu",
        # kwd={"AUTHOR": "Bo Zhang"}
    )
    rot90 = 1
    gain = dict(
        gain_unit=u.electron / u.adu,
        add_keyword={"AUTHOR": "Bo Zhang",
                     "GAINCORR": True},
    )

    bias_combine = dict(   # ???
        method="average"
    )
    settings = dict(
        rot=90,
        apwidth=15
    )
    apfind_kwds = dict(
        start_col=440,          # starting column
        max_drift=8,            # max drift pixel number
        max_apwidth=10,
        n_pix_goodap=100,
        n_adj=10,
        n_smooth=1,
        n_sep=3,
        c=5                     # gaussian smoothing (x, y)
    )
    scattered_light = dict(
        ap_width=10,
        method='median',
        method_kwargs=dict(selem=disk(5)),
        shrink=1.00,
    )
    normalization = dict(
        dwave=30,
        p=(1E-5, 1E-7),
        q=.5,
        ivar_block=None,
        eps=1e-10,
        rsv_frac=1.,
        n_jobs=10,
        verbose=False,
    )

    def __init__(self):
        pass


class Song(Table):
    """ represent SONG configuration """

    dirpath = ""
    cfg = Config()

    READOUT = None     # master
    BIAS = None        # master
    PATH_BIAS = []

    FLAT = None        # master
    FLAT_BIAS = None
    FLAT_NORM =None
    PATH_FLAT = []

    FLATI2 = None      # master
    FLATI2_BIAS = None
    PATH_FLATI2 = []

    THAR = None        # master
    THAR_BIAS = None
    PATH_THAR = []

    THARI2 = None      # master
    THARI2_BIAS = None
    PATH_THARI2 = []

    STAR = None
    TEST = None

    # aperture contents
    ap_comb = None
    ap_coefs = None
    ap_final = None

    def __init__(self, *args, **kwargs):
        super(Song, self).__init__(*args, **kwargs)
        # add other attributes here
        # balabala

    @staticmethod
    def _init_from_dir(dirpath, verbose=True):
        """ initiate from a directory path

        Parameters
        ----------
        dirpath: string
            a directory path

        Returns
        -------

        """
        assert os.path.exists(dirpath)
        s = Song(scan_files(dirpath, xdriftcol=False, verbose=verbose))
        s.dirpath = dirpath
        return s

    def select(self, colname="default", value="FLAT", method="random",
               n_images=10, return_colname=("fps"), verbose=False):
        """ select some images from list

        Parameters
        ----------
        colname: string
            name of the column that will be matched
        value:
            the specified value
        method: string, {"random", "top", "bottom"}
            the method adopted
        n_images:
            the number of images that will be selected
        return_colname:
            the name(s) of the column that will be returned
        verbose:
            if True, print resutl

        Returns
        -------
        the Song instance

        Examples
        --------
        >>> s.list_image(imagetp="STAR", kwds=["OBJECT"])

        """

        # get the colname of imagetype
        if colname is "default":
            colname = self.cfg.kwds["kw_imagetype"]

        # determine the matched images
        ind_match = np.where(self[colname] == value)[0]
        n_match = len(ind_match)
        if n_match < 1:
            print("@SONG: no images matched!")
            return None

        # determine the number of images to select
        n_images = np.min([n_match, n_images])

        # select according to method
        assert method in {"random", "top", "bottom", "all"}
        if method is "all":
            method = "top"
            n_images = n_match
        if method is "random":
            ind_rand = random_ind(n_match, n_images)
            if return_colname is "ind":
                result = ind_match[ind_rand]
            else:
                result = self[return_colname][ind_match[ind_rand]]
        elif method is "top":
            ind_rand = np.arange(0, n_images, dtype=int)
            if return_colname is "ind":
                result = ind_match[ind_rand]
            else:
                result = self[return_colname][ind_match[ind_rand]]
        elif method is "bottom":
            ind_rand = np.arange(n_match-n_images, n_match, dtype=int)
            if return_colname is "ind":
                result = ind_match[ind_rand]
            else:
                result = self[return_colname][ind_match[ind_rand]]

        # verbose
        if verbose:
            print("@SONG: these are all images selected")
            # here result is a Table
            result.pprint()
            # print("+ ----------------------------------------------")
            # print result
            # for r in result:
            #     print(r)
            # print("+ ----------------------------------------------")

        return result

    def ezselect(self, imgtype="FLAT", method="random", n_images=10,
                 verbose=False):
        return self.select(colname="default", value=imgtype, method=method,
                           n_images=n_images, return_colname="fps",
                           verbose=verbose)

    def list_image(self, imagetp="FLAT", kwds=None, max_print=None):
        list_image(self, imagetp=imagetp, return_col=None, kwds=kwds,
                   max_print=max_print)
        return

    # to add more info in summary
    @property
    def summary(self, colname_imagetype="IMAGETYP", return_data=False):
        """

        Parameters
        ----------
        colname_imagetype: string
            the keyword name for image type, default is "IMAGETYP"
        return_data: bool
            if True, return the summary data

        Returns
        -------
        unique images

        """

        u, uind, uinv, ucts = np.unique(self[colname_imagetype],
                                        return_counts=True, return_index=True,
                                        return_inverse=True)
        # print summary information
        print("=====================================================")
        print("[SUMMARY] {:s}".format(self.dirpath))
        print("=====================================================")
        for i in range(len(u)):
            print("{:10s} {:d}".format(u[i], ucts[i]))
        print("=====================================================")

        # return results
        if return_data:
            return u, uind, uinv, ucts

    def ezmaster(self, imgtype="BIAS", n_images=10, select="random",
                 method="mean", gain_corr=True):
        """

        Parameters
        ----------
        imgtype: string
            {"BIAS", "FLAT", "FLATI2", "THAR", "THARI2",
             "STAR", "STARI2", "TEST"}
        n_images: int
            number of images will be use
        select:
            scheme of selection
        method:
            method of combining
        gain_corr: bool
            if True, do gain correction

        Returns
        -------

        """

        assert select in {"random", "top", "bottom", "all"}
        assert imgtype in ALL_IMGTYPE

        fps = self.ezselect(imgtype=imgtype, method=select,
                                 n_images=n_images)
        if imgtype is "BIAS":
            print("@SONG: setting BIAS & READOUT ...")
            self.BIAS, self.READOUT = combine_image(fps, self.cfg,
                                                    method=method,
                                                    gain_corr=gain_corr)
            self.PATH_BIAS = fps

        else:
            print("@SONG: setting {:s} ...".format(imgtype))
            im = combine_image(fps, self.cfg, method=method,
                               gain_corr=gain_corr)[0]
            self.__setattr__(imgtype, im)
            self.__setattr__("PATH_{0}".format(imgtype), fps)

    def dump(self, fp):
        print("@SONG: save to {0} ...".format(fp))
        dump(self, fp)
        return

    @staticmethod
    def load(fp):
        print("@SONG: load from {0} ...".format(fp))
        return load(fp)
    
    def substract_bias(self, master="FLAT"):
        try:
            assert master in ALL_IMGTYPE
        except AssertionError as ae:
            raise(AssertionError(
                "@SONG: master type not valid!"))

        print("@SONG: substract bias for {0} ...".format(master))
        name = "{0}_BIAS".format(master)
        value = ccdproc.subtract_bias(self.__getattribute__(master), self.BIAS)
        self.__setattr__(name, value)

    def aptrace(self, imgs=None, n_jobs=10, verbose=False):

        # the default image used to trace apertures is FLAT_BIAS
        if imgs is None:
            imgs = self.FLAT_BIAS

        if isinstance(self.FLAT.data, np.ndarray) \
                or isinstance(s.FLAT, np.ndarray):
            imgs = [imgs]

        self.ap_comb = combine_apertures(
            imgs, n_jobs=n_jobs, find_aps_param_dict=self.cfg.apfind_kwds,
            verbose=verbose)
        self.ap_coefs, self.ap_final = group_apertures(
            self.ap_comb, start_col=1024, order_dist=7)

    def substract_scattered_light(self, img):
        img, sl = substract_scattered_light(
            img, self.ap_final, **self.cfg.scattered_light)
        return img, sl

    def apflatten(self, flat, ap_width=(-8, 8), **normalization):
        kwargs = self.cfg.normalization
        for k, v in normalization.items():
            kwargs[k] = v
        self.FLAT_NORM = apflatten(flat, self.ap_final, ap_width=ap_width,
                                   **kwargs)


def random_ind(n, m):
    """ from n choose m randomly """
    return np.argsort(np.random.rand(n,))[:m]


def list_image(t, imagetp="FLAT", return_col=None, kwds=None, max_print=None):
    """ list images with specified IMAGETYP value

    Examples
    --------
    >>> list_image(t2, imagetp="STAR", kwds=["OBJECT"])

    Parameters
    ----------
    t: Table
        catalog of files, generated by *scan_files*
    imagetp: string
        IMAGETYP value
    kwds: list
        optional. additional columns to be displayed
    max_print:
        max line number

    Returns
    -------

    """
    ind_mst = np.where(t["IMAGETYP"] == imagetp)[0]

    if max_print is not None:
        if max_print > len(ind_mst):
            max_print = len(ind_mst)
    else:
        max_print = len(ind_mst)

    print("@SONG: these are all images of type %s" % imagetp)
    print("+ --------------------------------------------------")
    if isinstance(kwds, str):
        kwds = [kwds]
    if kwds is None or kwds == "":
        for i in range(max_print):
            print("+ %04d - %s" % (i, t["fps"][ind_mst[i]]))
    else:
        assert isinstance(kwds, list) or isinstance(kwds, tuple)
        for kwd in kwds:
            try:
                assert kwd in t.colnames
            except AssertionError:
                print("kwd", kwd)
                raise AssertionError()

        for i in range(max_print):
            s = "+ %04d - %s" % (i, t["fps"][ind_mst[i]])
            for kwd in kwds:
                s += "  %s" % t[kwd][ind_mst[i]]
            print(s)
    print("+ --------------------------------------------------")

    if return_col is not None:
        return t[return_col][ind_mst]



# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Wed Jan  18 13:00:00 2017

Modifications
-------------
-

Aims
----
- the configuration class of twodspec

"""

DC_BIAS = dict(

)

DEFAULT_BACKGROUND = dict(
    # the extrapolation method, {median, average, percentile
    method='median',
    # the smoothing filter adopted {median, gaussian, box}
    smooth_filter='median',
    )

DEFAULT_EXTRACT = dict(
    # the extraction strategy {simple, optimal}
    method='simple',
)


class Config(object):
    background = {''}
    combine = {''}
    extract = {''}

    def __init__(self):
        pass

    def __repr__(self):
        pass


def test():
    cfg = Config()
    cfg.pprint()


if __name__ == "__main__":
    test()


# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Fri Feb 24 21:00:00 2017

Modifications
-------------
-

Aims
----
- utils to process master images

"""

import numpy as np

from twodspec import ccdproc_mod as ccdproc


def combine_image(fps, cfg, method='average', gain_corr=True):
    """ estimate bias & read noise """

    # read bias images
    image_list = [read_image(fp, cfg.read, gain_corr,
                             cfg.kwds['kw_pregain'], cfg.gain, cfg.rot90)
                 for fp in fps]
    image_array = np.array([im.data.astype(float) for im in image_list])
    image_unit = image_list[0].unit

    # estimate bias
    if method in {'average', 'mean'}:
        image_ = ccdproc.CCDData(np.mean(image_array, axis=0), unit=image_unit)
    elif method is 'median':
        image_ = ccdproc.CCDData(np.median(image_array, axis=0), unit=image_unit)
    else:
        raise(ValueError("@SONG: method is not valid!"))

    # estimate readout noise
    image_std = ccdproc.CCDData(np.std(image_array, axis=0), unit=image_unit)

    return image_, image_std


def read_image(fp, kwargs_read, gain_corr, kw_gain, kwargs_gain, rot90):
    im = ccdproc.CCDData.read(fp, **kwargs_read)
    if gain_corr:
        gain_value = im.header[kw_gain]
        im = ccdproc.gain_correct(im, gain=gain_value, **kwargs_gain)
    if rot90 is not None:
        im.rot90(rot90)
    return im



"""
an example of extracting 1d spectra for SONG

"""

#%%
%pylab qt
%matplotlib qt
%load_ext autoreload
%autoreload 2


from song import extract
from song import measure_xdrift
from song import ccdproc_mod as ccdproc
from song import utils
from song import Song
import hrs

from matplotlib import rcParams
rcParams.update({'font.size':20})

import numpy as np

#%%

reload(extract)
reload(measure_xdrift)

#%%

%%time
""" substantiate Song class """
s = Song._init_from_dir('/hydrogen/song/star_spec/20170216/night/raw/', verbose=False)
#s.select(value='FLAT', method='random',  verbose=False, return_colname='ind')
#s.select(value='FLAT', method='random', verbose=False, return_colname=('fps'))

s.summary

#fps = s.ezselect('STAR', 'random', 10)
#s.list_image('STAR', ('OBJECT', 'MJD-MID'))


#%%

""" process MASTERs """
from song import master

s.ezmaster(imgtype='BIAS', n_images=10, select='random', method='mean', gain_corr=True)
s.ezmaster(imgtype='FLAT', n_images=10, select='random', method='mean', gain_corr=True)
s.ezmaster(imgtype='FLATI2', n_images=10, select='random', method='mean', gain_corr=True)
s.ezmaster(imgtype='THAR', n_images=10, select='random', method='mean', gain_corr=True)

s.substract_bias(master='FLAT')
s.substract_bias(master='FLATI2')
s.substract_bias(master='THAR')

#%%
""" find apertures """

s.aptrace(s.FLAT_BIAS, n_jobs=1, verbose=False)
#s.ap_coefs
#s.ap_comb.shape
#s.ap_final.shape


#%%
""" scattered light """
from skimage.morphology import disk
s.cfg.scattered_light['ap_width']=13
s.cfg.scattered_light['method']='gaussian'
s.cfg.scattered_light['method_kwargs']=dict(sigma=(15, 5))
s.cfg.scattered_light['shrink']=1.0

#s.cfg.scattered_light['method']='median'
#s.cfg.scattered_light['method_kwargs']=dict(selem=disk(5))

flat_bias_sl, sl = s.substract_scattered_light(s.FLAT_BIAS.data)

#%%
""" apflatten """

s.apflatten(s.FLAT, ap_width=(-8, 8))

from twodspec.aperture import get_dispersion_slice
line = get_dispersion_slice(s.FLAT_BIAS, s.ap_final, i_order=22, offset=1)
x = np.arange(len(line))

#from scipy.interpolate import splrep, splev
#tck = splrep(x, line, s=0.5)
#line_ = splev(x, tck)
#plot(x,line)
#plot(x, line_)

from twodspec.aperture import apflatten
flat_norm = apflatten(s.FLAT.data, ap_uorder_interp, ap_width=(-8, 8), **s.cfg.normalization)
imshow(flat_norm, vmin=.99, vmax=1.01)

flat_norm_ccd = ccdproc.CCDData(flat_norm, unit='electron')
flat_norm_ccd.write('/home/cham/Desktop/flat_norm.fits')

from slam.normalization import normalize_spectrum
cont = normalize_spectrum(x, line, x[[0, -1]], 30, p=(1E-5, 1E-7), q=0.5, ivar=None, eps=1e-10, rsv_frac=1.)[1]
plot(x, line)
plot(x, cont)

plot(s.FLAT_BIAS.data[:, 1000])
plot(sl[:,1000])
s.ezextract()


from twodspec.aperture import substract_scattered_light
substract_scattered_light(s.FLAT_BIAS, s.ap_final, ap_width=10, method='median', method_kwargs=dict(selem=disk(5)), shrink=1.0)

from twodspec.aperture import combine_apertures
find_aps_param_dict = dict(start_col=440, max_drift=8, max_apwidth=10,
                           n_pix_goodap=100, n_adj=10, n_smooth=1, n_sep=3, c=5)


ap_comb = hrs.combine_apertures([ccdproc.subtract_bias(s.FLAT, s.BIAS)],
                                n_jobs=1, find_aps_param_dict=find_aps_param_dict)
cheb_coefs, ap_uorder_interp = hrs.group_apertures(ap_comb, start_col=1024, order_dist=7)




""" extract 1d FLAT & STAR & THAR """
flat_bias_sl = hrs.substract_scattered_light(flat_bias, ap_uorder_interp, ap_width=10, shrink=.85)
flat1d = hrs.extract_1dspec(flat_bias_sl, ap_uorder_interp, ap_width=8)[0]

thar_bias_sl = hrs.substract_scattered_light(thar_bias, ap_uorder_interp, ap_width=10, shrink=.85)
thar1d = hrs.extract_1dspec(thar_bias, ap_uorder_interp, ap_width=8)[0]




from scipy.ndimage.filters import median_filter
d = np.arange(25.,).reshape(5, 5)
median_filter(d, size=(2,2))

#%%

imshow(s.FLAT.data/flat_norm, interpolation='nearest')
plot(s.ap_final.T, 'w')



""" process FLAT """
flat_fps = s.ezselect('FLAT', 'random', 10)
flat, flat_std = master.combine(flat_fps, s, True, 'mean')



from twodspec import ccdproc_mod as ccdproc
def read_image(fp):
    im = ccdproc.CCDData.read(fps[0], unit='adu')
    
plt.imshow(im, interpolation='nearest')

im = ccdproc.CCDData.read(fps[0], unit='adu', author='bozhang')
im.header
    
#%%

# specify directory
dp = '/hydrogen/song/star_spec/20170102/night/raw/'
dp = '/hydrogen/song/star_spec/20170216/night/raw/'

# scan fits files
t = utils.scan_files(dp)

t.show_in_browser()

# measure cross-order drift
t2, fig = measure_xdrift.check_xdrift(t)
f90 = utils.scan_flux90(t, n_jobs=20, unit='adu')
plot(f90[t['IMAGETYP']=='FLAT'])
plot(f90[t['IMAGETYP']=='STAR'])


""" BIAS """
bias = extract.produce_master(t2, method='average', imagetp='BIAS', slc=slice(0, 10)).rot90(1)

""" FLAT """
flat = extract.produce_master(t2, method='average', imagetp='FLAT', slc=slice(0, 100)).rot90(1)
flat_bias = ccdproc_mod.subtract_bias(flat, bias)

""" THAR """
thar = extract.produce_master(t2, method='median', imagetp='THAR', slc=slice(0, 1)).rot90(1)
thar_bias = ccdproc_mod.subtract_bias(thar, bias)



""" list all STAR images """
extract.list_image(t2, imagetp='STAR', kwds=['OBJECT'])



#%%
""" VCUT """
flat216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/20161110006.fits', unit='adu')
thar216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/201611100039.fits', unit='adu')
star216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/201611100052.fits', unit='adu')

fig = figure(figsize=(15, 10))
ax = fig.add_subplot(211)
ax.plot(flat_bias[:,1024])
ax.plot(thar_bias[:,1024])
ax.plot(star_bias[:,1024])
ax.legend(['flat', 'thar', 'star'])
ax.set_title('SONG')

ax = fig.add_subplot(212)
ax.plot(flat216[:,2048])
ax.plot(thar216[:,2048])
ax.plot(star216[:,2048])
ax.legend(['flat', 'thar', 'star'])
ax.set_title('2.16m HRS')

#%%


# #################################### #
#    find apertures
# #################################### #

#extract.list_image(t, 'FLAT')

""" find apertures """
find_aps_param_dict = dict(start_col=440, max_drift=8, max_apwidth=10,
                           n_pix_goodap=100, n_adj=10, n_smooth=1, n_sep=3, c=5)
ap_comb = hrs.combine_apertures([flat_bias], n_jobs=1, find_aps_param_dict=find_aps_param_dict)
cheb_coefs, ap_uorder_interp = hrs.group_apertures(ap_comb, start_col=1024, order_dist=7)

""" extract 1d FLAT & STAR & THAR """
flat_bias_sl = hrs.substract_scattered_light(flat_bias, ap_uorder_interp, ap_width=10, shrink=.85)
flat1d = hrs.extract_1dspec(flat_bias_sl, ap_uorder_interp, ap_width=8)[0]

thar_bias_sl = hrs.substract_scattered_light(thar_bias, ap_uorder_interp, ap_width=10, shrink=.85)
thar1d = hrs.extract_1dspec(thar_bias, ap_uorder_interp, ap_width=8)[0]



# #################################### #
#    determine wave_init
# #################################### #

# laod template thar
thar_temp_path = '/home/cham/PycharmProjects/hrs/song/calibration/thar_template/thar_template.fits'
wave_temp, thar_temp, order_temp = hrs.load_thar_temp(thar_temp_path)

# fix thar
thar1d_fixed = hrs.fix_thar_sat_neg(thar1d, arm=30, sat_count=60000*15)
thar_temp_fixed = hrs.fix_thar_sat_neg(thar_temp, arm=30, sat_count=60000*15)


""" initial estimation of wavelength """
ytrim = (wave_temp.shape[0]*np.array([.2, .8])).astype(int)
xtrim = (wave_temp.shape[1]*np.array([.2, .8])).astype(int)
shift, corr2d = hrs.thar_corr2d(thar1d_fixed-np.percentile(thar1d_fixed, 50),
                                thar_temp-np.percentile(thar_temp, 50),
                                xtrim=xtrim, ytrim=ytrim, 
                                y_shiftmax = 3, x_shiftmax=20)

imshow(corr2d, interpolation='nearest')
print ("@Cham: the shift is ", shift)

wave_init = hrs.interpolate_wavelength_shift(wave_temp, shift, thar_temp, thar1d_fixed)
order_init = hrs.interpolate_order_shift(order_temp, shift, thar1d_fixed)


# #################################### #
#    determine wave_init
# #################################### #

""" list all STAR images """
extract.list_image(t2, imagetp='STAR', kwds=['OBJECT'])


""" STAR """
fig = figure()

for i in range(63):
    star = extract.produce_master(t2, method='median', imagetp='STAR', slc=slice(i, i+1)).rot90(1)
    star_bias = ccdproc_mod.subtract_bias(star, bias)
    
    star_bias_sl = hrs.substract_scattered_light(star_bias, ap_uorder_interp, ap_width=10, shrink=.85)
    star1d = hrs.extract_1dspec(star_bias_sl, ap_uorder_interp, ap_width=8)[0]
    
    star1d_dblz = star1d/flat1d
    
    #plot(wave_init.T, (flat1d/np.percentile(flat1d, 90, axis=1)[:, None]).T, 'b')
    #plot(wave_init.T, (star1d/np.percentile(star1d, 90, axis=1)[:, None]).T, 'g')
    plot(wave_init.T, (star1d_dblz/np.percentile(star1d_dblz, 90, axis=1)[:, None]).T*0.9+i, 'r')



fig = figure()
plot(wave_temp.T, thar_temp.T, 'r')
plot(wave_init.T, thar1d.T, 'b')


apflat = ccdproc_mod.CCDData.read('/home/cham/s2_2017-01-02T19-42-30_.fits', unit='adu')
figure(), imshow(apflat, vmin=0.5, vmax=1.5)


np.savetxt






plot(wave_init.T, star1d_dblz.T, 'r')

np.savetxt('/home/cham/Desktop/thar1d.dat', thar1d)

thar_threshold = .90

for i in range(thar1d.shape[0]):
    plot(thar1d[i]/np.max(thar1d[i])+i)
    ind_gt_thar_thre = np.where(thar1d[i]>np.percentile(thar1d[i], thar_threshold*100))[0]
    plot(ind_gt_thar_thre, (thar1d[i]/np.max(thar1d[i]))[ind_gt_thar_thre]+i, 'r.')
    
    
hist(thar1d[0], 1000)
plot(thar1d[0])

"""  """
thar_temp_path = '/home/cham/PycharmProjects/hrs/hrs/calibration/thar_template/thar_temp_w20160120022t.fits'
wave_temp, thar_temp, order_temp = load_thar_temp(thar_temp_path)




#%%
for i in range(star1d_dblz.shape[0]):
    plot(star1d_dblz[i, :]/np.median(star1d_dblz[i, :])+i*1)


#%%
imshow(log10(thar_bias), interpolation='nearest')
plot(ap_comb.T, 'k-')
plot(ap_uorder_interp.T, 'w-')

